@@ -101,31 +101,40 @@ typedef enum {
     /* Is an STable. */
     MVM_CF_STABLE = 2,
 
+    /* Is a heap-promoted call frame. */
+    MVM_CF_FRAME = 4,
+
     /* Has already been seen once in GC nursery. */
-    MVM_CF_NURSERY_SEEN = 4,
+    MVM_CF_NURSERY_SEEN = 8,
 
     /* Has been promoted to the old generation. */
-    MVM_CF_SECOND_GEN = 8,
+    MVM_CF_SECOND_GEN = 16,
 
     /* Is shared - that is, more than one thread knows about it. */
-    MVM_CF_SHARED = 16,
+    MVM_CF_SHARED = 32,
 
     /* Has already been added to the gen2 aggregates pointing to nursery
      * objects list. */
-    MVM_CF_IN_GEN2_ROOT_LIST = 32,
+    MVM_CF_IN_GEN2_ROOT_LIST = 64,
 
     /* A full GC run has found this object to be live. */
-    MVM_CF_GEN2_LIVE = 64,
+    MVM_CF_GEN2_LIVE = 128,
 
     /* This object in fromspace is live with a valid forwarder. */
     /* TODO - should be possible to use the same bit for this and GEN2_LIVE. */
-    MVM_CF_FORWARDER_VALID = 128,
+    MVM_CF_FORWARDER_VALID = 256,
 
     /* Have we allocated memory to store a serialization index? */
-    MVM_CF_SERIALZATION_INDEX_ALLOCATED = 256,
+    MVM_CF_SERIALZATION_INDEX_ALLOCATED = 512,
 
     /* Have we arranged a persistent object ID for this object? */
-    MVM_CF_HAS_OBJECT_ID = 512
+    MVM_CF_HAS_OBJECT_ID = 1024,
+
+    /* Have we flagged this object as something we must never repossess? */
+    /* Note: if you're hunting for a flag, some day in the future when we
+     * have used them all, this one is easy enough to eliminate by having the
+     * tiny number of objects marked this way in a remembered set. */
+    MVM_CF_NEVER_REPOSSESS = 2048
 } MVMCollectableFlags;
 
 #ifdef MVM_USE_OVERFLOW_SERIALIZATION_INDEX
@@ -143,18 +152,12 @@ struct MVMSerializationIndex {
  * type object.
  */
 struct MVMCollectable {
-    /* Identifier of the thread that currently owns the object, if any. If the
-     * object is unshared, then this is always the creating thread. If it is
-     * shared then it's whoever currently holds the mutex on it, or 0 if there
-     * is no held mutex. */
-    MVMuint32 owner;
-
-    /* Collectable flags (see MVMCollectableFlags). */
-    MVMuint16 flags;
-
-    /* Object size, in bytes. */
-    MVMuint16 size;
-
+    /* Put this union first, as these pointers/indexes are relatively "cold",
+       whereas "flags" is accessed relatively frequently, as are the fields
+       that follow in the structures into which MVMCollectable is embedded.
+       Shrinking the size of the active part of the structure slightly
+       increases the chance that it fits into the CPU's L1 cache, which is a
+       "free" performance win. */
     union {
         /* Forwarding pointer, for copying/compacting GC purposes. */
         MVMCollectable *forwarder;
@@ -175,6 +178,18 @@ struct MVMCollectable {
         /* Used to chain STables queued to be freed. */
         MVMSTable *st;
     } sc_forward_u;
+
+    /* Identifier of the thread that currently owns the object, if any. If the
+     * object is unshared, then this is always the creating thread. If it is
+     * shared then it's whoever currently holds the mutex on it, or 0 if there
+     * is no held mutex. */
+    MVMuint32 owner;
+
+    /* Collectable flags (see MVMCollectableFlags). */
+    MVMuint16 flags;
+
+    /* Object size, in bytes. */
+    MVMuint16 size;
 };
 #ifdef MVM_USE_OVERFLOW_SERIALIZATION_INDEX
 #  define MVM_DIRECT_SC_IDX_SENTINEL 0xFFFF
@@ -216,6 +231,11 @@ struct MVMBoolificationSpec {
     MVMuint32  mode;
 };
 
+/* Constant for incrementing the type cache ID for new STables. This leaves
+ * the lowest bits free for caches to attach flags (of note, the multi
+ * dispatch cache). */
+#define MVM_TYPE_CACHE_ID_INCR 256
+
 /* S-table, representing a meta-object/representation pairing. Note that the
  * items are grouped in hope that it will pack decently and do decently in
  * terms of cache lines. */
@@ -325,6 +345,14 @@ struct MVMSTable {
     /* Also info we need to lazily deserialize the method cache. */
     MVMuint32                method_cache_offset;
     MVMSerializationContext *method_cache_sc;
+
+    /* A string associated with this STable for debugging purposes.
+     * Usually the name of the class this belongs to. */
+    char *debug_name;
+
+    /* If this STable is currently in the process of being repossessed. Used
+     * to trigger clearup of memory pre-repossession. */
+    MVMuint8 being_repossessed;
 };
 
 /* The representation operations table. Note that representations are not
@@ -386,6 +414,16 @@ struct MVMREPROps_Boxing {
     MVMString * (*get_str) (MVMThreadContext *tc, MVMSTable *st,
         MVMObject *root, void *data);
 
+    /* Used with boxing. Sets an unsinged integer value, for representations
+     *  that can hold one. */
+    void (*set_uint) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data, MVMuint64 value);
+
+    /* Used with boxing. Gets an unsigned integer value, for representations
+     * that can hold one. */
+    MVMuint64 (*get_uint) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data);
+
     /* Some objects serve primarily as boxes of others, inlining them. This gets
      * gets the reference to such things, using the representation ID to distinguish
      * them. */
@@ -411,10 +449,6 @@ struct MVMREPROps_Positional {
     void (*set_elems) (MVMThreadContext *tc, MVMSTable *st,
         MVMObject *root, void *data, MVMuint64 count);
 
-    /* Returns a true value of the specified index exists, and a false one if not. */
-    MVMint64 (*exists_pos) (MVMThreadContext *tc, MVMSTable *st,
-        MVMObject *root, void *data, MVMint64 index);
-
     /* Pushes the specified value onto the array. */
     void (*push) (MVMThreadContext *tc, MVMSTable *st,
         MVMObject *root, void *data, MVMRegister value, MVMuint16 kind);
@@ -439,6 +473,29 @@ struct MVMREPROps_Positional {
         MVMObject *root, void *data, MVMObject *target_array,
         MVMint64 offset, MVMuint64 elems);
 
+    /* Multi-dimensional array read. */
+    void (*at_pos_multidim) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data, MVMint64 num_indices,
+        MVMint64 *indices, MVMRegister *result, MVMuint16 kind);
+
+    /* Multi-dimensional array write. */
+    void (*bind_pos_multidim) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data, MVMint64 num_indices,
+        MVMint64 *indices, MVMRegister value, MVMuint16 kind);
+
+    /* Gets the number of dimensions along with a C-level array of them. The
+     * second two parameters are "out"s. The caller must not mutate dimensions,
+     * nor persist it such that it lasts longer than the next VM safepoint. */
+    void (*dimensions) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data, MVMint64 *num_dimensions,
+        MVMint64 **dimensions);
+
+    /* Sets the number of dimensions. The caller is responsible for freeing
+     * the array passed in dimensions. */
+    void (*set_dimensions) (MVMThreadContext *tc, MVMSTable *st,
+        MVMObject *root, void *data, MVMint64 num_dimensions,
+        MVMint64 *dimensions);
+
     /* Gets the STable representing the declared element type. */
     MVMStorageSpec (*get_elem_storage_spec) (MVMThreadContext *tc, MVMSTable *st);
 };
@@ -568,9 +625,13 @@ struct MVMREPROps {
     /* The representation's ID. */
     MVMuint32 ID;
 
-    /* Does this representation reference frames (either MVMStaticFrame or
-     * MVMFrame)? */
-    MVMuint32 refs_frames;
+    /* Optional API, for representations that allocate additonal memory and
+     * want to report its size for debugging purposes. */
+    MVMuint64 (*unmanaged_size) (MVMThreadContext *tc, MVMSTable *st, void *data);
+
+    /* Optional API to describe references to other Collectables either by
+     * index or by name, i.E. names of attributes or lexicals. */
+    void (*describe_refs) (MVMThreadContext *tc, MVMHeapSnapshotState *ss, MVMSTable *st, void *data);
 };
 
 /* Various handy macros for getting at important stuff. */
@@ -582,8 +643,8 @@ struct MVMREPROps {
 #define IS_CONCRETE(o)   (!(((MVMObject *)o)->header.flags & MVM_CF_TYPE_OBJECT))
 
 /* Some functions related to 6model core functionality. */
-MVMObject * MVM_6model_get_how(MVMThreadContext *tc, MVMSTable *st);
-MVMObject * MVM_6model_get_how_obj(MVMThreadContext *tc, MVMObject *st);
+MVM_PUBLIC MVMObject * MVM_6model_get_how(MVMThreadContext *tc, MVMSTable *st);
+MVM_PUBLIC MVMObject * MVM_6model_get_how_obj(MVMThreadContext *tc, MVMObject *obj);
 void MVM_6model_find_method(MVMThreadContext *tc, MVMObject *obj, MVMString *name, MVMRegister *res);
 MVM_PUBLIC MVMObject * MVM_6model_find_method_cache_only(MVMThreadContext *tc, MVMObject *obj, MVMString *name);
 MVMint32 MVM_6model_find_method_spesh(MVMThreadContext *tc, MVMObject *obj, MVMString *name,
@@ -596,3 +657,4 @@ MVMint64 MVM_6model_try_cache_type_check(MVMThreadContext *tc, MVMObject *obj, M
 void MVM_6model_invoke_default(MVMThreadContext *tc, MVMObject *invokee, MVMCallsite *callsite, MVMRegister *args);
 void MVM_6model_stable_gc_free(MVMThreadContext *tc, MVMSTable *st);
 MVMuint64 MVM_6model_next_type_cache_id(MVMThreadContext *tc);
+void MVM_6model_never_repossess(MVMThreadContext *tc, MVMObject *obj);
