@@ -44,8 +44,19 @@ struct MVMSerializationRoot {
     MVMint32  num_repos;
     char     *repos_table;
 
-    /* Array of STRINGs. */
+    /* The number of parameterized type intern entries, and the data segment
+     * containing them. */
+    MVMint32  num_param_interns;
+    char     *param_interns_data;
+
+    /* Array of strings making up the string heap we are constructing. If we
+     * are reading, this will either contain a string heap array or be NULL
+     * and the next field will be set. */
     MVMObject *string_heap;
+
+    /* The compilation unit whose string heap we will use to locate strings.
+     * This must be set of string_heap about is set to NULL. */
+    MVMCompUnit *string_comp_unit;
 };
 
 /* Indexes the deserializer still has to work on. */
@@ -66,11 +77,13 @@ struct MVMSerializationReader {
     MVMint32 stables_data_offset;
     MVMint32 objects_data_offset;
     MVMint32 contexts_data_offset;
+    MVMint32 param_interns_data_offset;
 
     /* Limits up to where we can read stables, objects and contexts data. */
     char *stables_data_end;
     char *objects_data_end;
     char *contexts_data_end;
+    char *param_interns_data_end;
 
     /* Where to find details related to the current buffer we're reading from:
      * the buffer pointer itself, the current offset and the amount that is
@@ -112,9 +125,13 @@ struct MVMSerializationWriter {
     /* Serialization root data. */
     MVMSerializationRoot root;
 
-    /* The code refs and contexts lists we're working through/adding to. */
-    MVMObject  *codes_list;
-    MVMObject  *contexts_list;
+    /* The code refs we're working through/adding to. */
+    MVMObject *codes_list;
+
+    /* Frames that we're to serialize, along with memory management. */
+    MVMFrame **contexts_list;
+    MVMuint32 num_contexts;
+    MVMuint32 alloc_contexts;
 
     /* Current position in the stables, objects and contexts lists. */
     MVMint64 stables_list_pos;
@@ -135,12 +152,14 @@ struct MVMSerializationWriter {
     MVMuint32 contexts_table_alloc;
     MVMuint32 contexts_data_alloc;
     MVMuint32 repos_table_alloc;
+    MVMuint32 param_interns_data_alloc;
 
     /* Current offsets for the data chunks (also correspond to the amount of
      * data written in to them). */
     MVMuint32 stables_data_offset;
     MVMuint32 objects_data_offset;
     MVMuint32 contexts_data_offset;
+    MVMuint32 param_interns_data_offset;
 
     /* Where to find details related to the current buffer we're writing in
      * to: the buffer pointer itself, the current offset and the amount that
@@ -166,15 +185,14 @@ MVMObject * MVM_serialization_demand_code(MVMThreadContext *tc, MVMSerialization
 void MVM_serialization_finish_deserialize_method_cache(MVMThreadContext *tc, MVMSTable *st);
 
 /* Reader/writer functions. */
+MVMint64 MVM_serialization_read_int64(MVMThreadContext *tc, MVMSerializationReader *reader);
 MVMint64 MVM_serialization_read_int(MVMThreadContext *tc, MVMSerializationReader *reader);
-MVMint64 MVM_serialization_read_varint(MVMThreadContext *tc, MVMSerializationReader *reader);
 MVMnum64 MVM_serialization_read_num(MVMThreadContext *tc, MVMSerializationReader *reader);
 MVMString * MVM_serialization_read_str(MVMThreadContext *tc, MVMSerializationReader *reader);
 MVMObject * MVM_serialization_read_ref(MVMThreadContext *tc, MVMSerializationReader *reader);
 MVMSTable * MVM_serialization_read_stable_ref(MVMThreadContext *tc, MVMSerializationReader *reader);
 void MVM_serialization_force_stable(MVMThreadContext *tc, MVMSerializationReader *reader, MVMSTable *st);
 void MVM_serialization_write_int(MVMThreadContext *tc, MVMSerializationWriter *writer, MVMint64 value);
-void MVM_serialization_write_varint(MVMThreadContext *tc, MVMSerializationWriter *writer, MVMint64 value);
 void MVM_serialization_write_num(MVMThreadContext *tc, MVMSerializationWriter *writer, MVMnum64 value);
 void MVM_serialization_write_str(MVMThreadContext *tc, MVMSerializationWriter *writer, MVMString *value);
 void MVM_serialization_write_ref(MVMThreadContext *tc, MVMSerializationWriter *writer, MVMObject *ref);
