@@ -54,9 +54,6 @@ struct MVMThreadContext {
     /* Thread object representing the thread. */
     MVMThread *thread_obj;
 
-    /* The frame lying at the base of the current thread. */
-    MVMFrame *thread_entry_frame;
-
     /* Pointer to where the interpreter's current opcode is stored. */
     MVMuint8 **interp_cur_op;
 
@@ -71,9 +68,18 @@ struct MVMThreadContext {
      * is stored. */
     MVMCompUnit **interp_cu;
 
+    /* First call stack memory region, so we can traverse them for cleanup. */
+    MVMCallStackRegion *stack_first;
+
+    /* Current call stack region, which the next frame will be allocated in. */
+    MVMCallStackRegion *stack_current;
+
     /* The frame we're currently executing. */
     MVMFrame *cur_frame;
 
+    /* The frame lying at the base of the current thread. */
+    MVMFrame *thread_entry_frame;
+
     /* libuv event loop */
     uv_loop_t *loop;
 
@@ -85,10 +91,13 @@ struct MVMThreadContext {
     /* Linked list of exception handlers that we're currently executing, topmost
      * one first in the list. */
     MVMActiveHandler *active_handlers;
-    
+
     /* Result object of the last-run exception handler. */
     MVMObject *last_handler_result;
 
+    /* Last payload made available in a payload-goto exception handler. */
+    MVMObject *last_payload;
+
     /* Mutex that must be released if we throw an exception. Used in places
      * like I/O, which grab a mutex but may throw an exception. */
     uv_mutex_t *ex_release_mutex;
@@ -112,9 +121,11 @@ struct MVMThreadContext {
     MVMuint32 gc_promoted_bytes;
 
     /* Memory buffer pointing to the last thing we serialized, intended to go
-     * into the next compilation unit we write. */
+     * into the next compilation unit we write. Also the serialized string
+     * heap, which will be used to seed the compilation unit string heap. */
     MVMint32      serialized_size;
     char         *serialized;
+    MVMObject    *serialized_string_heap;
 
     /* Temporarily rooted objects. This is generally used by code written in
      * C that wants to keep references to objects. Since those may change
@@ -153,18 +164,10 @@ struct MVMThreadContext {
     MVMuint32        gc_work_size;
     MVMuint32        gc_work_count;
 
-    /* Pool table of chains of frames for each static frame. */
-    MVMFrame **frame_pool_table;
-
     /* Pool of Lexotics for various static frames, held per thread since the
-     * result being returned is per thread. Indexes are same as used in the
-     * frame_pool_table above. */
+     * result being returned is per thread. */
     MVMLexotic **lexotic_cache;
-
-    /* Size of the frame pool table and lexotic cache, so they can grow on
-     * demand. */
-    MVMuint32 frame_pool_table_size;
-    MVMuint32 lexotic_cache_size;
+    MVMuint32    lexotic_cache_size;
 
     /* Serialization context write barrier disabled depth (anything non-zero
      * means disabled). */
@@ -174,8 +177,10 @@ struct MVMThreadContext {
      * index 0. */
     MVMObject     *compiling_scs;
 
-    /* Dispatcher set for next invocation to take. */
+    /* Dispatcher for next invocation that matches _for to take. If _for is
+     * NULL then anything matches. */
     MVMObject     *cur_dispatcher;
+    MVMObject     *cur_dispatcher_for;
 
     /* Cache of native code callback data. */
     MVMNativeCallbackCacheHead *native_callback_cache;
@@ -198,8 +203,26 @@ struct MVMThreadContext {
     MVMint64 *nfa_longlit;
     MVMint64  nfa_longlit_len;
 
+    /* Memory for doing multi-dim indexing with late-bound dimension counts. */
+    MVMint64 *multi_dim_indices;
+    MVMint64  num_multi_dim_indices;
+
+    /* The number of locks the thread is holding. */
+    MVMint64 num_locks;
+
     /* Profiling data collected for this thread, if profiling is on. */
     MVMProfileThreadData *prof_data;
+
+    /* Frame sequence numbers in order to cheaply identify the place of a frame
+     * in the call stack */
+    MVMint32 current_frame_nr;
+    MVMint32 next_frame_nr;
+
+#if MVM_GC_DEBUG
+    /* Whether we are currently in the specializer. Used to catch GC runs that
+     * take place at times they never should. */
+    MVMint32 in_spesh;
+#endif
 };
 
 MVMThreadContext * MVM_tc_create(MVMInstance *instance);
