@@ -101,6 +101,7 @@ typedef enum {
     MVM_JIT_LITERAL_PTR,
     MVM_JIT_REG_STABLE,
     MVM_JIT_REG_OBJBODY,
+    MVM_JIT_DATA_LABEL,
 } MVMJitArgType;
 
 struct MVMJitCallArg {
@@ -154,12 +155,18 @@ struct MVMJitInvoke {
 struct MVMJitJumpList {
     MVMint64 num_labels;
     MVMint16 reg;
-    // labels of the goto's / jump instructions themselves
+    /* labels of the goto's / jump instructions themselves */
     MVMint32 *in_labels;
-    // labels the goto's jump to
+    /* labels the goto's jump to */
     MVMint32 *out_labels;
 };
 
+struct MVMJitData {
+    MVMint32 label;
+    void     *data;
+    size_t    size;
+};
+
 /* Node types */
 typedef enum {
     MVM_JIT_NODE_PRIMITIVE,
@@ -170,11 +177,12 @@ typedef enum {
     MVM_JIT_NODE_INVOKE,
     MVM_JIT_NODE_JUMPLIST,
     MVM_JIT_NODE_CONTROL,
+    MVM_JIT_NODE_DATA,
 } MVMJitNodeType;
 
 struct MVMJitNode {
-    MVMJitNode   * next; // linked list
-    MVMJitNodeType type; // tag
+    MVMJitNode   * next; /* linked list */
+    MVMJitNodeType type; /* tag */
     union {
         MVMJitPrimitive prim;
         MVMJitCallC     call;
@@ -184,6 +192,7 @@ struct MVMJitNode {
         MVMJitInvoke    invoke;
         MVMJitJumpList  jumplist;
         MVMJitControl   control;
+        MVMJitData      data;
     } u;
 };
 
