@@ -91,6 +91,8 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 
 /* Headers for various other data structures and APIs. */
 #include "6model/6model.h"
+#include "gc/collect.h"
+#include "gc/debug.h"
 #include "gc/wb.h"
 #include "core/threadcontext.h"
 #include "core/instance.h"
@@ -100,8 +102,8 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "core/exceptions.h"
 #include "core/alloc.h"
 #include "core/frame.h"
+#include "core/callstack.h"
 #include "core/validation.h"
-#include "core/compunit.h"
 #include "core/bytecode.h"
 #include "core/bytecodedump.h"
 #include "core/ops.h"
@@ -126,14 +128,15 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "6model/sc.h"
 #include "6model/serialization.h"
 #include "6model/parametric.h"
+#include "core/compunit.h"
 #include "gc/gen2.h"
 #include "gc/allocation.h"
 #include "gc/worklist.h"
-#include "gc/collect.h"
 #include "gc/orchestrate.h"
 #include "gc/roots.h"
 #include "gc/objectid.h"
 #include "gc/finalize.h"
+#include "core/regionalloc.h"
 #include "spesh/dump.h"
 #include "spesh/graph.h"
 #include "spesh/codegen.h"
@@ -147,6 +150,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "spesh/threshold.h"
 #include "spesh/inline.h"
 #include "spesh/osr.h"
+#include "spesh/lookup.h"
 #include "strings/normalize.h"
 #include "strings/decode_stream.h"
 #include "strings/ascii.h"
@@ -184,6 +188,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "profiler/instrument.h"
 #include "profiler/log.h"
 #include "profiler/profile.h"
+#include "profiler/heapsnapshot.h"
 #include "instrument/crossthreadwrite.h"
 
 MVMObject *MVM_backend_config(MVMThreadContext *tc);
@@ -199,6 +204,12 @@ MVM_PUBLIC void MVM_vm_set_exec_name(MVMInstance *instance, const char *exec_nam
 MVM_PUBLIC void MVM_vm_set_prog_name(MVMInstance *instance, const char *prog_name);
 MVM_PUBLIC void MVM_vm_set_lib_path(MVMInstance *instance, int count, const char **lib_path);
 
+#if defined(__s390__)
+AO_t AO_fetch_compare_and_swap_emulation(volatile AO_t *addr, AO_t old_val, AO_t new_val);
+# define AO_fetch_compare_and_swap_full(addr, old, newval) \
+    AO_fetch_compare_and_swap_emulation(addr, old, newval)
+#endif
+
 /* Returns original. Use only on AO_t-sized values (including pointers). */
 #define MVM_incr(addr) AO_fetch_and_add1_full((volatile AO_t *)(addr))
 #define MVM_decr(addr) AO_fetch_and_sub1_full((volatile AO_t *)(addr))
