@@ -1,3 +1,6 @@
+#define MVMPhiNodeCacheSize             48
+#define MVMPhiNodeCacheSparseBegin      32
+
 /* Top level of a spesh graph, representing a particular static frame (and
  * potentially having others inlined into it). */
 struct MVMSpeshGraph {
@@ -41,9 +44,8 @@ struct MVMSpeshGraph {
     /* Number of log-based guards we have. */
     MVMint32 num_log_guards;
 
-    /* Memory blocks we allocate to store spesh nodes, and which we free along
-     * with the graph. Contains a link to previous blocks. */
-    MVMSpeshMemBlock *mem_block;
+    /* Region allocator for spesh nodes */
+    MVMRegionAlloc region_alloc;
 
     /* Values placed in spesh slots. */
     MVMCollectable **spesh_slots;
@@ -93,25 +95,16 @@ struct MVMSpeshGraph {
     MVMuint16          num_temps;
     MVMuint16          alloc_temps;
     MVMSpeshTemporary *temps;
-};
 
-/* The default allocation chunk size for memory blocks used to store spesh
- * graph nodes. Power of two is best. */
-#define MVM_SPESH_MEMBLOCK_SIZE 32768
+    /* We need to create new MVMOpInfo structs for each number of
+     * arguments a PHI node can take. We cache them here, so that we
+     * allocate fewer of them across our spesh alloc blocks.
+     */
+    MVMOpInfo *phi_infos;
 
-/* A block of bump-pointer allocated memory. */
-struct MVMSpeshMemBlock {
-    /* The memory buffer itself. */
-    char *buffer;
-
-    /* Current allocation position. */
-    char *alloc;
-
-    /* Allocation limit. */
-    char *limit;
-
-    /* Previous, now full, memory block. */
-    MVMSpeshMemBlock *prev;
+    /* If this graph was formed from a spesh candidate rather than an
+     * original static frame, the candidate will be stored here. */
+    MVMSpeshCandidate *cand;
 };
 
 /* A temporary register, added to support transformations. */
@@ -159,6 +152,14 @@ struct MVMSpeshBB {
      * the block identifier in dominance computation and for debug output. */
     MVMint32 idx;
 
+    /* The block's reverse post-order index, assinged when computing
+     * dominance. */
+    MVMint32 rpo_idx;
+
+    /* We cache the instruction pointer of the very first instruction so that
+     * we can output a line number for every BB */
+    MVMuint32 initial_pc;
+
     /* Is this block an inlining of another one? */
     MVMint32 inlined;
 };
@@ -233,11 +234,13 @@ struct MVMSpeshAnn {
 #define MVM_SPESH_ANN_DEOPT_INLINE  8
 #define MVM_SPESH_ANN_DEOPT_OSR     9
 
-/* Functions to create/destory the spesh graph. */
-MVMSpeshGraph * MVM_spesh_graph_create(MVMThreadContext *tc, MVMStaticFrame *sf, MVMuint32 cfg_only);
+/* Functions to create/destroy the spesh graph. */
+MVMSpeshGraph * MVM_spesh_graph_create(MVMThreadContext *tc, MVMStaticFrame *sf,
+    MVMuint32 cfg_only, MVMuint32 insert_object_nulls);
 MVMSpeshGraph * MVM_spesh_graph_create_from_cand(MVMThreadContext *tc, MVMStaticFrame *sf,
     MVMSpeshCandidate *cand, MVMuint32 cfg_only);
 MVMSpeshBB * MVM_spesh_graph_linear_prev(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *search);
 void MVM_spesh_graph_mark(MVMThreadContext *tc, MVMSpeshGraph *g, MVMGCWorklist *worklist);
 void MVM_spesh_graph_destroy(MVMThreadContext *tc, MVMSpeshGraph *g);
 MVM_PUBLIC void * MVM_spesh_alloc(MVMThreadContext *tc, MVMSpeshGraph *g, size_t bytes);
+MVMOpInfo *get_phi(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint32 nrargs);
