@@ -17,6 +17,7 @@ typedef struct MVMCallCapture MVMCallCapture;
 typedef struct MVMCallCaptureBody MVMCallCaptureBody;
 typedef struct MVMCallsite MVMCallsite;
 typedef struct MVMCallsiteInterns MVMCallsiteInterns;
+typedef struct MVMCallStackRegion MVMCallStackRegion;
 typedef struct MVMCFunction MVMCFunction;
 typedef struct MVMCFunctionBody MVMCFunctionBody;
 typedef struct MVMCode MVMCode;
@@ -31,6 +32,8 @@ typedef struct MVMContainerRegistry MVMContainerRegistry;
 typedef struct MVMContext MVMContext;
 typedef struct MVMContextBody MVMContextBody;
 typedef struct MVMContinuationTag MVMContinuationTag;
+typedef struct MVMDecoder MVMDecoder;
+typedef struct MVMDecoderBody MVMDecoderBody;
 typedef struct MVMDLLRegistry MVMDLLRegistry;
 typedef struct MVMDLLSym MVMDLLSym;
 typedef struct MVMDLLSymBody MVMDLLSymBody;
@@ -39,6 +42,8 @@ typedef struct MVMExceptionBody MVMExceptionBody;
 typedef struct MVMExtOpRecord MVMExtOpRecord;
 typedef struct MVMExtOpRegistry MVMExtOpRegistry;
 typedef struct MVMExtRegistry MVMExtRegistry;
+typedef struct MVMRegionAlloc MVMRegionAlloc;
+typedef struct MVMRegionBlock MVMRegionBlock;
 typedef struct MVMFixedSizeAlloc MVMFixedSizeAlloc;
 typedef struct MVMFixedSizeAllocFreeListEntry MVMFixedSizeAllocFreeListEntry;
 typedef struct MVMFixedSizeAllocSafepointFreeListEntry MVMFixedSizeAllocSafepointFreeListEntry;
@@ -74,7 +79,7 @@ typedef struct MVMNFAStateInfo MVMNFAStateInfo;
 typedef struct MVMNFGState MVMNFGState;
 typedef struct MVMNFGSynthetic MVMNFGSynthetic;
 typedef struct MVMNFGTrieNode MVMNFGTrieNode;
-typedef struct MVMNGFTrieNodeEntry MVMNGFTrieNodeEntry;
+typedef struct MVMNFGTrieNodeEntry MVMNFGTrieNodeEntry;
 typedef struct MVMNativeCall MVMNativeCall;
 typedef struct MVMNativeCallBody MVMNativeCallBody;
 typedef struct MVMNativeRef MVMNativeRef;
@@ -104,7 +109,7 @@ typedef struct MVMCUnionNameMap MVMCUnionNameMap;
 typedef struct MVMCUnionREPRData MVMCUnionREPRData;
 typedef struct MVMMultiCache MVMMultiCache;
 typedef struct MVMMultiCacheBody MVMMultiCacheBody;
-typedef struct MVMMultiArityCache MVMMultiArityCache;
+typedef struct MVMMultiCacheNode MVMMultiCacheNode;
 typedef struct MVMMultiDimArray MVMMultiDimArray;
 typedef struct MVMMultiDimArrayBody MVMMultiDimArrayBody;
 typedef struct MVMMultiDimArrayREPRData MVMMultiDimArrayREPRData;
@@ -171,6 +176,7 @@ typedef struct MVMSpeshInline MVMSpeshInline;
 typedef struct MVMSTable MVMSTable;
 typedef struct MVMStaticFrame MVMStaticFrame;
 typedef struct MVMStaticFrameBody MVMStaticFrameBody;
+typedef struct MVMStaticFrameInstrumentation MVMStaticFrameInstrumentation;
 typedef struct MVMStorageSpec MVMStorageSpec;
 typedef struct MVMString MVMString;
 typedef struct MVMStringBody MVMStringBody;
@@ -183,6 +189,7 @@ typedef struct MVMThreadBody MVMThreadBody;
 typedef struct MVMThreadContext MVMThreadContext;
 typedef struct MVMUnicodeNamedValue MVMUnicodeNamedValue;
 typedef struct MVMUnicodeNameRegistry MVMUnicodeNameRegistry;
+typedef struct MVMUnicodeGraphemeNameRegistry MVMUnicodeGraphemeNameRegistry;
 typedef struct MVMUninstantiable MVMUninstantiable;
 typedef struct MVMWorkThread MVMWorkThread;
 typedef struct MVMIOOps MVMIOOps;
@@ -220,9 +227,19 @@ typedef struct MVMJitGuard MVMJitGuard;
 typedef struct MVMJitInvoke MVMJitInvoke;
 typedef struct MVMJitJumpList MVMJitJumpList;
 typedef struct MVMJitControl MVMJitControl;
+typedef struct MVMJitData MVMJitData;
 typedef struct MVMJitCode MVMJitCode;
 typedef struct MVMProfileThreadData MVMProfileThreadData;
 typedef struct MVMProfileGC MVMProfileGC;
 typedef struct MVMProfileCallNode MVMProfileCallNode;
 typedef struct MVMProfileAllocationCount MVMProfileAllocationCount;
 typedef struct MVMProfileContinuationData MVMProfileContinuationData;
+typedef struct MVMHeapSnapshotCollection MVMHeapSnapshotCollection;
+typedef struct MVMHeapSnapshot MVMHeapSnapshot;
+typedef struct MVMHeapSnapshotType MVMHeapSnapshotType;
+typedef struct MVMHeapSnapshotStaticFrame MVMHeapSnapshotStaticFrame;
+typedef struct MVMHeapSnapshotCollectable MVMHeapSnapshotCollectable;
+typedef struct MVMHeapSnapshotReference MVMHeapSnapshotReference;
+typedef struct MVMHeapSnapshotState MVMHeapSnapshotState;
+typedef struct MVMHeapSnapshotWorkItem MVMHeapSnapshotWorkItem;
+typedef struct MVMHeapSnapshotSeen MVMHeapSnapshotSeen;
